#include<iostream>
#include<cmath>
#include<limits>

template<typename T>
T f(const T& a, const T& x, const unsigned int b, const T& c) { 
  return a*pow(x,b)+c; 
}

template<typename T>
T dfdx_symbolically(const T& a, const T& x, const unsigned int b, const T&) { 
  return b*a*pow(x,b-1); 
}

template<typename T>
T dfdx_numerically(const T& a, const T& x, const unsigned int b, const T& c) { 
  T h= x==0 ? sqrt(std::numeric_limits<T>::epsilon())
            : sqrt(std::numeric_limits<T>::epsilon())*fabs(x);
  return (f(a,x+h,b,c)-f(a,x-h,b,c))/(2*h);
}

template<typename T>
T newton(const T& a, T x, const unsigned int b, const T& c, const T& eps, 
		T (*f)(const T&, const T&, const unsigned int, const T&), 
		T (*dfdx)(const T&, const T&, const unsigned int, const T&)) {
  static int i=1;
  T r=f(a,x,b,c);
  if (fabs(r)<=eps) return x;
  x-=r/dfdx(a,x,b,c);
  std::cout << i++ << ": " << x << std::endl;
  return newton(a,x,b,c,eps,f,dfdx);
}

int main() {
  using T=double; using namespace std;
  const T a=2, c=-42; int b=3;
  T x; cout << "x="; cin >> x;
  T z=newton(a,x,b,c,sqrt(numeric_limits<T>::epsilon()),f,dfdx_numerically);
  // T z=newton(a,x,b,c,sqrt(numeric_limits<T>::epsilon()),f,dfdx_symbolically);
  cout << "f(" << z << ")=" << f(a,z,b,c) << endl;
  return 0;
}
